@@ -14,11 +14,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
-
-
-
-
 import os
 
 import pygame
@@ -156,7 +151,8 @@ class Sprite(object):
         created.
 
         """
-        print('Creating sprite "{0}"'.format(name))
+        if sge.DEBUG:
+            print('Creating sprite "{0}"'.format(name))
         self.name = name
 
         self._transparent = None
@@ -168,10 +164,6 @@ class Sprite(object):
         fname_frames = []
         fname_strip = None
 
-        print("Current image directories:")
-        for d in sge.image_directories:
-            print(os.path.normpath(os.path.abspath(d)))
-
         if name is not None:
             for path in sge.image_directories:
                 if os.path.isdir(path):
@@ -204,7 +196,8 @@ class Sprite(object):
                     img = pygame.image.load(fname_single)
                     self._baseimages.append(img)
                 except pygame.error:
-                    print("Ignored {0}; not a valid image.".format(fname_single))
+                    if sge.DEBUG:
+                        print("Ignored {0}; not a valid image.".format(fname_single))
 
             if not self._baseimages and any(fname_frames):
                 # Load the multiple images
@@ -213,7 +206,8 @@ class Sprite(object):
                         try:
                             self._baseimages.append(pygame.image.load(fname))
                         except pygame.error:
-                            print("Ignored {0}; not a valid image.".format(fname))
+                            if sge.DEBUG:
+                                print("Ignored {0}; not a valid image.".format(fname))
 
             if not self._baseimages and fname_strip:
                 # Load the strip (sprite sheet)
@@ -238,9 +232,13 @@ class Sprite(object):
                         img = sheet.subsurface(rect)
                         self._baseimages.append(img)
                 except pygame.error:
-                    print("Ignored {0}; not a valid image.".format(fname_strip))
+                    if sge.DEBUG:
+                        print("Ignored {0}; not a valid image.".format(fname_strip))
 
             if not self._baseimages:
+                print("Directories searched:")
+                for d in sge.image_directories:
+                    print(os.path.normpath(os.path.abspath(d)))
                 msg = 'Files for sprite name "{0}" not found.'.format(name)
                 raise IOError(msg)
         else:
@@ -260,7 +258,8 @@ class Sprite(object):
             img = pygame.Surface((width, height), pygame.SRCALPHA)
             img.fill(pygame.Color(0, 0, 0, 0))
             self._baseimages.append(img)
-            print("renamed to {0}".format(self.name))
+            if sge.DEBUG:
+                print("renamed to {0}".format(self.name))
 
         if width is None:
             width = 1
