@@ -14,11 +14,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
-
-
-
-
 import os
 import math
 import weakref
@@ -49,6 +44,11 @@ class StellarClass(object):
             Sprite object or None when read, it can also be set to the
             ID of a sprite.
         visible: Whether or not the object should be drawn.
+        active: Whether or not the object should be active.  If this is
+            False, all normal occurances (including events) will not
+            execute.  Only collision events, destroy events, and events
+            with names that start with "event_inactive_" will be
+            executed.
         detects_collisions: Whether or not the object should be involved
             in collisions.
         bbox_x: The horizontal location of the top-left corner of the
@@ -126,6 +126,7 @@ class StellarClass(object):
         event_mouse_button_release: Mouse button release event.
         event_joystick_axis_move: Joystick axis move event.
         event_joystick_hat_move: Joystick HAT move event.
+        event_joystick_trackball_move: Joystick trackball move event.
         event_joystick_button_press: Joystick button press event.
         event_joystick_button_release: Joystick button release event.
         event_close: Close event (e.g. close button).
@@ -135,6 +136,21 @@ class StellarClass(object):
         event_collision_top: Top collision event.
         event_collision_bottom: Bottom collision event.
 
+    The following alternative events are executed when the object is
+    inactive (i.e. the attribute ``active`` is set to False) in place of
+    the corresponding normal events:
+        event_inactive_step
+        event_inactive_key_press
+        event_inactive_key_release
+        event_inactive_mouse_move
+        event_inactive_mouse_button_press
+        event_inactive_mouse_button_release
+        event_inactive_joystick_axis_move
+        event_inactive_joystick_hat_move
+        event_inactive_joystick_trackball_move
+        event_inactive_joystick_button_press
+        event_inactive_joystick_button_release
+
     The following alternative events are executed when the game is
     paused in place of the corresponding normal events:
         event_paused_key_press
@@ -144,6 +160,7 @@ class StellarClass(object):
         event_paused_mouse_button_release
         event_paused_joystick_axis_move
         event_paused_joystick_hat_move
+        event_paused_joystick_trackball_move
         event_paused_joystick_button_press
         event_paused_joystick_button_release
 
@@ -342,12 +359,12 @@ class StellarClass(object):
             self._frame_time = None
 
     def __init__(self, x, y, z, id_=None, sprite=None, visible=True,
-                 detects_collisions=True, bbox_x=None, bbox_y=None,
-                 bbox_width=None, bbox_height=None, collision_ellipse=False,
-                 collision_precise=False, xvelocity=0, yvelocity=0,
-                 image_index=0, image_fps=0, image_xscale=1, image_yscale=1,
-                 image_rotation=0, image_alpha=255, image_blend=None,
-                 **kwargs):
+                 active=True, detects_collisions=True, bbox_x=None,
+                 bbox_y=None, bbox_width=None, bbox_height=None,
+                 collision_ellipse=False, collision_precise=False,
+                 xvelocity=0, yvelocity=0, image_index=0, image_fps=0,
+                 image_xscale=1, image_yscale=1, image_rotation=0,
+                 image_alpha=255, image_blend=None, **kwargs):
         """Create a new StellarClass object.
 
         Arguments set the properties of the object.  See
@@ -377,6 +394,7 @@ class StellarClass(object):
         self._image_index = image_index
         self.sprite = sprite
         self.visible = visible
+        self.active = active
         self.detects_collisions = detects_collisions
         if self.sprite is not None:
             sprite_bbox_x = self.sprite.bbox_x
@@ -469,14 +487,16 @@ class StellarClass(object):
                 if isinstance(obj(), other):
                     others.append(obj())
 
-        if x is None:
-            x = self.x
-        if y is None:
-            y = self.y
-
         # Change x and y to be offset values; these are easier to use.
-        x -= self.x
-        y -= self.y
+        if x is not None:
+            x -= self.x
+        else:
+            x = 0
+
+        if y is not None:
+            y -= self.y
+        else:
+            y = 0
 
         for other in others:
             if (self.collision_precise or self.collision_ellipse or
@@ -569,8 +589,8 @@ class StellarClass(object):
     def event_step(self, time_passed):
         """Step event.
 
-        Called once each frame.  ``time_passed`` is the number of
-        milliseconds that have passed during the last frame.
+        See the documentation for sge.Game.event_step for more
+        information.
 
         """
         pass
@@ -595,7 +615,8 @@ class StellarClass(object):
     def event_key_press(self, key):
         """Key press event.
 
-        ``key`` is the key that was pressed.
+        See the documentation for sge.Game.event_key_press for more
+        information.
 
         """
         pass
@@ -603,7 +624,8 @@ class StellarClass(object):
     def event_key_release(self, key):
         """Key release event.
 
-        ``key`` is the key that was pressed.
+        See the documentation for sge.Game.event_key_release for more
+        information.
 
         """
         pass
@@ -611,7 +633,8 @@ class StellarClass(object):
     def event_mouse_move(self, x, y):
         """Mouse move event.
 
-        ``x`` and ``y`` indicate the relative movement of the mouse.
+        See the documentation for sge.Game.event_mouse_move for more
+        information.
 
         """
         pass
@@ -619,9 +642,8 @@ class StellarClass(object):
     def event_mouse_button_press(self, button):
         """Mouse button press event.
 
-        ``button`` is the number of the mouse button that was pressed;
-        these numbers may vary by implementation, so MOUSE_BUTTON_*
-        constants should be used.
+        See the documentation for sge.Game.event_mouse_button_press for
+        more information.
 
         """
         pass
@@ -629,9 +651,8 @@ class StellarClass(object):
     def event_mouse_button_release(self, button):
         """Mouse button release event.
 
-        ``button`` is the number of the mouse button that was released;
-        these numbers may vary by implementation, so MOUSE_BUTTON_*
-        constants should be used.
+        See the documentation for sge.Game.event_mouse_button_release
+        for more information.
 
         """
         pass
@@ -639,14 +660,8 @@ class StellarClass(object):
     def event_joystick_axis_move(self, joystick, axis, value):
         """Joystick axis move event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``axis`` is the number of the axis, where 0 is the
-        first axis.  ``value`` is the tilt of the axis, where 0 is in
-        the center, -1 is tilted all the way to the left or up, and 1 is
-        tilted all the way to the right or down.
-
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        See the documentation for sge.Game.event_joystick_axis_move for
+        more information.
 
         """
         pass
@@ -654,14 +669,17 @@ class StellarClass(object):
     def event_joystick_hat_move(self, joystick, hat, x, y):
         """Joystick HAT move event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``hat`` is the number of the HAT, where 0 is the
-        first HAT.  ``x`` and ``y`` indicate the position of the HAT,
-        where 0 is in the center, -1 is left or up, and 1 is right or
-        down.
+        See the documentation for sge.Game.event_joystick_hat_move for
+        more information.
 
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        """
+        pass
+
+    def event_joystick_trackball_move(self, joystick, ball, x, y):
+        """Joystick trackball move event.
+
+        See the documentation for sge.Game.event_joystick_trackball_move
+        for more information.
 
         """
         pass
@@ -669,12 +687,8 @@ class StellarClass(object):
     def event_joystick_button_press(self, joystick, button):
         """Joystick button press event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``button`` is the number of the button pressed, where
-        0 is the first button.
-
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        See the documentation for sge.Game.event_joystick_button_press
+        for more information.
 
         """
         pass
@@ -682,12 +696,8 @@ class StellarClass(object):
     def event_joystick_button_release(self, joystick, button):
         """Joystick button release event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``button`` is the number of the button pressed, where
-        0 is the first button.
-
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        See the documentation for sge.Game.event_joystick_button_release
+        for more information.
 
         """
         pass
@@ -712,10 +722,120 @@ class StellarClass(object):
         """Bottom collision event."""
         self.event_collision(other)
 
+    def event_inactive_step(self, time_passed):
+        """Step event when this object is inactive.
+
+        See the documentation for sge.Game.event_step for more
+        information.  The object is considered to be inactive when its
+        ``active`` attribute is False.
+
+        """
+
+    def event_inactive_key_press(self, key):
+        """Key press event when this object is inactive.
+
+        See the documentation for sge.Game.event_key_press for more
+        information.  The object is considered to be inactive when its
+        ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_key_release(self, key):
+        """Key release event when this object is inactive.
+
+        See the documentation for sge.Game.event_key_release for more
+        information.  The object is considered to be inactive when its
+        ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_mouse_move(self, x, y):
+        """Mouse move event when this object is inactive.
+
+        See the documentation for sge.Game.event_mouse_move for more
+        information.  The object is considered to be inactive when its
+        ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_mouse_button_press(self, button):
+        """Mouse button press event when this object is inactive.
+
+        See the documentation for sge.Game.event_mouse_button_press for
+        more information.  The object is considered to be inactive when
+        its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_mouse_button_release(self, button):
+        """Mouse button release event when this object is inactive.
+
+        See the documentation for sge.Game.event_mouse_button_release
+        for more information.  The object is considered to be inactive
+        when its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_joystick_axis_move(self, joystick, axis, value):
+        """Joystick axis move event when this object is inactive.
+
+        See the documentation for sge.Game.event_joystick_axis_move for
+        more information.  The object is considered to be inactive when
+        its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_joystick_hat_move(self, joystick, hat, x, y):
+        """Joystick HAT move event when this object is inactive.
+
+        See the documentation for sge.Game.event_joystick_hat_move for
+        more information.  The object is considered to be inactive when
+        its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_joystick_trackball_move(self, joystick, ball, x, y):
+        """Joystick trackball move event when this object is inactive.
+
+        See the documentation for sge.Game.event_joystick_trackball_move
+        for more information.  The object is considered to be inactive
+        when its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_joystick_button_press(self, joystick, button):
+        """Joystick button press event when this object is inactive.
+
+        See the documentation for sge.Game.event_joystick_button_press
+        for more information.  The object is considered to be inactive
+        when its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_joystick_button_release(self, joystick, button):
+        """Joystick button release event when this object is inactive.
+
+        See the documentation for sge.Game.event_joystick_button_release
+        for more information.  The object is considered to be inactive
+        when its ``active`` attribute is False.
+
+        """
+        pass
+
     def event_paused_key_press(self, key):
         """Key press event when paused.
 
-        See StellarClass.event_key_press.__doc__ for more information.
+        See the documentation for sge.Game.event_key_press for more
+        information.
 
         """
         pass
@@ -723,7 +843,8 @@ class StellarClass(object):
     def event_paused_key_release(self, key):
         """Key release event when paused.
 
-        See StellarClass.event_key_release.__doc__ for more information.
+        See the documentation for sge.Game.event_key_release for more
+        information.
 
         """
         pass
@@ -731,7 +852,8 @@ class StellarClass(object):
     def event_paused_mouse_move(self, x, y):
         """Mouse move event when paused.
 
-        See StellarClass.event_mouse_move.__doc__ for more information.
+        See the documentation for sge.Game.event_mouse_move for more
+        information.
 
         """
         pass
@@ -739,8 +861,8 @@ class StellarClass(object):
     def event_paused_mouse_button_press(self, button):
         """Mouse button press event when paused.
 
-        See StellarClass.event_mouse_button_press.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_mouse_button_press for
+        more information.
 
         """
         pass
@@ -748,8 +870,8 @@ class StellarClass(object):
     def event_paused_mouse_button_release(self, button):
         """Mouse button release event when paused.
 
-        See StellarClass.event_mouse_button_release.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_mouse_button_release
+        for more information.
 
         """
         pass
@@ -757,8 +879,8 @@ class StellarClass(object):
     def event_paused_joystick_axis_move(self, joystick, axis, value):
         """Joystick axis move event when paused.
 
-        See StellarClass.event_joystick_axis_move.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_axis_move for
+        more information.
 
         """
         pass
@@ -766,8 +888,17 @@ class StellarClass(object):
     def event_paused_joystick_hat_move(self, joystick, hat, x, y):
         """Joystick HAT move event when paused.
 
-        See StellarClass.event_joystick_hat_move.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_hat_move for
+        more information.
+
+        """
+        pass
+
+    def event_paused_joystick_trackball_move(self, joystick, ball, x, y):
+        """Joystick trackball move event when paused.
+
+        See the documentation for sge.Game.event_joystick_trackball_move
+        for more information.
 
         """
         pass
@@ -775,8 +906,8 @@ class StellarClass(object):
     def event_paused_joystick_button_press(self, joystick, button):
         """Joystick button press event when paused.
 
-        See StellarClass.event_joystick_button_press.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_button_press
+        for more information.
 
         """
         pass
@@ -784,8 +915,8 @@ class StellarClass(object):
     def event_paused_joystick_button_release(self, joystick, button):
         """Joystick button release event when paused.
 
-        See StellarClass.event_joystick_button_release.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_button_release
+        for more information.
 
         """
         pass
@@ -815,7 +946,7 @@ class StellarClass(object):
             self._anim_count %= self._frame_time
 
         # Alarms
-        for a in self._alarms:
+        for a in list(self._alarms.keys()):
             self._alarms[a] -= delta_mult
             if self._alarms[a] <= 0:
                 del self._alarms[a]
