@@ -121,6 +121,7 @@ class Game(object):
         event_mouse_button_release: Mouse button release event.
         event_joystick_axis_move: Joystick axis move event.
         event_joystick_hat_move: Joystick HAT move event.
+        event_joystick_trackball_move: Joystick trackball move event.
         event_joystick_button_press: Joystick button press event.
         event_joystick_button_release: Joystick button release event.
         event_close: Close event (e.g. close button).  It is always
@@ -141,6 +142,7 @@ class Game(object):
         event_paused_mouse_button_release
         event_paused_joystick_axis_move
         event_paused_joystick_hat_move
+        event_paused_joystick_trackball_move
         event_paused_joystick_button_press
         event_paused_joystick_button_release
         event_paused_close
@@ -321,34 +323,46 @@ class Game(object):
 
         """
         if self._running:
+            if sge.DEBUG:
+                print("Restarting the game.")
+
             for room in self.rooms:
                 room._reset()
 
             self.rooms[0].start()
         else:
+            if sge.DEBUG:
+                print("Starting the game.")
             self._running = True
             self._background_changed = True
             self.event_game_start()
             self.rooms[0].start()
             background = None
             numviews = 0
+            _fps_time = 0
             self._clock.tick()
 
             while self._running:
                 # Pygame events
                 for event in pygame.event.get():
                     if event.type == pygame.KEYDOWN:
-                        k = sge.KEYNAMES[event.key]
+                        k = sge.KEY_NAMES[event.key]
                         self.event_key_press(k)
                         self.current_room.event_key_press(k)
                         for obj in self.current_room.objects:
-                            obj.event_key_press(k)
+                            if obj.active:
+                                obj.event_key_press(k)
+                            else:
+                                obj.event_inactive_key_press(k)
                     elif event.type == pygame.KEYUP:
-                        k = sge.KEYNAMES[event.key]
+                        k = sge.KEY_NAMES[event.key]
                         self.event_key_release(k)
                         self.current_room.event_key_release(k)
                         for obj in self.current_room.objects:
-                            obj.event_key_release(k)
+                            if obj.active:
+                                obj.event_key_release(k)
+                            else:
+                                obj.event_inactive_key_release(k)
                     elif event.type == pygame.MOUSEMOTION:
                         mx, my = event.pos
                         self.mouse.mouse_x = mx - self._x
@@ -356,84 +370,96 @@ class Game(object):
                         self.event_mouse_move(*event.rel)
                         self.current_room.event_mouse_move(*event.rel)
                         for obj in self.current_room.objects:
-                            obj.event_mouse_move(*event.rel)
+                            if obj.active:
+                                obj.event_mouse_move(*event.rel)
+                            else:
+                                obj.event_inactive_mouse_move(*event.rel)
                     elif event.type == pygame.MOUSEBUTTONDOWN:
-                        self.event_mouse_button_press(event.button)
-                        self.current_room.event_mouse_button_press(
-                            event.button)
+                        b = sge.MOUSE_BUTTON_NAMES[event.button]
+                        self.event_mouse_button_press(b)
+                        self.current_room.event_mouse_button_press(b)
                         for obj in self.current_room.objects:
-                            obj.event_mouse_button_press(event.button)
+                            if obj.active:
+                                obj.event_mouse_button_press(b)
+                            else:
+                                obj.event_inactive_mouse_button_press(b)
                     elif event.type == pygame.MOUSEBUTTONUP:
-                        self.event_mouse_button_release(event.button)
-                        self.current_room.event_mouse_button_release(
-                            event.button)
+                        b = sge.MOUSE_BUTTON_NAMES[event.button]
+                        self.event_mouse_button_release(b)
+                        self.current_room.event_mouse_button_release(b)
                         for obj in self.current_room.objects:
-                            obj.event_mouse_button_release(event.button)
+                            if obj.active:
+                                obj.event_mouse_button_release(b)
+                            else:
+                                obj.event_inactive_mouse_button_release(b)
                     elif event.type == pygame.JOYAXISMOTION:
                         self.event_joystick_axis_move(event.joy, event.axis,
                                                       event.value)
                         self.current_room.event_joystick_axis_move(
                             event.joy, event.axis, event.value)
                         for obj in self.current_room.objects:
-                            obj.event_joystick_axis_move(event.joy, event.axis,
-                                                         event.value)
-                    elif event.type == pygame.JOYBALLMOTION:
-                        # Limited support for trackballs by pretending
-                        # they're axes.  Since they're acting like axes,
-                        # they must be in the range [-1,1] unless the
-                        # special variable real_trackballs is True.
-                        n = (self._joysticks[event.joy].get_numaxes() +
-                             2 * event.ball)
-                        
-                        if sge.real_trackballs:
-                            xvalue = event.rel[0]
-                            yvalue = event.rel[1]
-                        else:
-                            xvalue = min(max(-1, event.rel[0]), 1)
-                            yvalue = min(max(-1, event.rel[1]), 1)
-
-                        # x-axis
-                        self.event_joystick_axis_move(event.joy, n, xvalue)
-                        self.current_room.event_joystick_axis_move(
-                            event.joy, n, xvalue)
-                        for obj in self.current_room.objects:
-                            obj.event_joystick_axis_move(event.joy, n, xvalue)
-                        
-                        # y-axis
-                        self.event_joystick_axis_move(event.joy, n + 1, yvalue)
-                        self.current_room.event_joystick_axis_move(
-                            event.joy, n + 1, yvalue)
-                        for obj in self.current_room.objects:
-                            obj.event_joystick_axis_move(event.joy, n + 1,
-                                                         yvalue)
+                            if obj.active:
+                                obj.event_joystick_axis_move(
+                                    event.joy, event.axis, event.value)
+                            else:
+                                obj.event_inactive_joystick_axis_move(
+                                    event.joy, event.axis, event.value)
                     elif event.type == pygame.JOYHATMOTION:
                         self.event_joystick_hat_move(event.joy, event.hat,
                                                      *event.value)
                         self.current_room.event_joystick_hat_move(
                             event.joy, event.hat, *event.value)
                         for obj in self.current_room.objects:
-                            obj.event_joystick_hat_move(event.joy, event.hat,
-                                                        *event.value)
+                            if obj.active:
+                                obj.event_joystick_hat_move(
+                                    event.joy, event.hat, *event.value)
+                            else:
+                                obj.event_inactive_joystick_hat_move(
+                                    event.joy, event.hat, *event.value)
+                    elif event.type == pygame.JOYBALLMOTION:
+                        self.event_joystick_trackball_move(
+                            event.joy, event.ball, *event.rel)
+                        self.current_room.event_joystick_trackball_move(
+                            event.joy, event.ball, *event.rel)
+                        for obj in self.current_room.objects:
+                            if obj.active:
+                                obj.event_joystick_trackball_move(
+                                    event.joy, event.ball, *event.rel)
+                            else:
+                                obj.event_inactive_joystick_trackball_move(
+                                    event.joy, event.ball, *event.rel)
                     elif event.type == pygame.JOYBUTTONDOWN:
                         self.event_joystick_button_press(event.joy,
                                                          event.button)
                         self.current_room.event_joystick_button_press(
                             event.joy, event.button)
                         for obj in self.current_room.objects:
-                            obj.event_joystick_button_press(event.joy,
-                                                            event.button)
+                            if obj.active:
+                                obj.event_joystick_button_press(event.joy,
+                                                                event.button)
+                            else:
+                                obj.event_inactive_joystick_button_press(
+                                    event.joy, event.button)
                     elif event.type == pygame.JOYBUTTONUP:
                         self.event_joystick_button_release(event.joy,
                                                            event.button)
                         self.current_room.event_joystick_button_release(
                             event.joy, event.button)
                         for obj in self.current_room.objects:
-                            obj.event_joystick_button_release(event.joy,
-                                                              event.button)
+                            if obj.active:
+                                obj.event_joystick_button_release(event.joy,
+                                                                  event.button)
+                            else:
+                                obj.event_inactive_joystick_button_release(
+                                    event.joy, event.button)
                     elif event.type == pygame.QUIT:
+                        if sge.DEBUG:
+                            print('Quit requested by the system.')
                         self.current_room.event_close()
                         self.event_close()
                     elif event.type == pygame.VIDEORESIZE:
+                        if sge.DEBUG:
+                            print('Video resize detected.')
                         self._window_width = event.w
                         self._window_height = event.h
                         self._set_mode()
@@ -448,6 +474,13 @@ class Game(object):
                     time_passed = 1000 / self.fps
                     delta_mult = 1
 
+                if sge.DEBUG:
+                    _fps_time += real_time_passed
+                    if _fps_time >= 250:
+                        _fps_time = 0
+                        self.window_text = "FPS: {0}; Delta: {1}".format(
+                            int(1000 / real_time_passed), delta_mult)
+
                 # Step events
                 self.event_step(real_time_passed)
                 self.current_room.event_step(real_time_passed)
@@ -458,8 +491,11 @@ class Game(object):
 
                 # Update objects (including mouse)
                 for obj in self.current_room.objects:
-                    obj._update(time_passed, delta_mult)
-                    obj.event_step(real_time_passed)
+                    if obj.active:
+                        obj._update(time_passed, delta_mult)
+                        obj.event_step(real_time_passed)
+                    else:
+                        obj.event_inactive_step(real_time_passed)
 
                 # Music control
                 if self._music is not None:
@@ -534,6 +570,8 @@ class Game(object):
             self.event_game_end()
             pygame.quit()
             sge.game = None
+            if sge.DEBUG:
+                print("Game ended normally.")
 
     def end(self):
         """Properly end the game."""
@@ -591,35 +629,37 @@ class Game(object):
             # Events
             for event in pygame.event.get():
                 if event.type == pygame.KEYDOWN:
-                    k = sge.KEYNAMES[event.key]
+                    k = sge.KEY_NAMES[event.key]
                     self.event_paused_key_press(k)
                     self.current_room.event_paused_key_press(k)
                     for obj in self.current_room.objects:
                         obj.event_paused_key_press(k)
                 elif event.type == pygame.KEYUP:
-                    k = sge.KEYNAMES[event.key]
+                    k = sge.KEY_NAMES[event.key]
                     self.event_paused_key_release(k)
                     self.current_room.event_paused_key_release(k)
                     for obj in self.current_room.objects:
                         obj.event_paused_key_release(k)
                 elif event.type == pygame.MOUSEMOTION:
-                    self.mouse.mouse_x, self.mouse.mouse_y = event.pos
+                    mx, my = event.pos
+                    self.mouse.mouse_x = mx - self._x
+                    self.mouse.mouse_y = my - self._y
                     self.event_paused_mouse_move(*event.rel)
                     self.current_room.event_paused_mouse_move(*event.rel)
                     for obj in self.current_room.objects:
                         obj.event_paused_mouse_move(*event.rel)
                 elif event.type == pygame.MOUSEBUTTONDOWN:
-                    self.event_paused_mouse_button_press(event.button)
-                    self.current_room.event_paused_mouse_button_press(
-                        event.button)
+                    b = sge.MOUSE_BUTTON_NAMES[event.button]
+                    self.event_paused_mouse_button_press(b)
+                    self.current_room.event_paused_mouse_button_press(b)
                     for obj in self.current_room.objects:
-                        obj.event_paused_mouse_button_press(event.button)
+                        obj.event_paused_mouse_button_press(b)
                 elif event.type == pygame.MOUSEBUTTONUP:
-                    self.event_paused_mouse_button_release(event.button)
-                    self.current_room.event_paused_mouse_button_release(
-                        event.button)
+                    b = sge.MOUSE_BUTTON_NAMES[event.button]
+                    self.event_paused_mouse_button_release(b)
+                    self.current_room.event_paused_mouse_button_release(b)
                     for obj in self.current_room.objects:
-                        obj.event_paused_mouse_button_release(event.button)
+                        obj.event_paused_mouse_button_release(b)
                 elif event.type == pygame.JOYAXISMOTION:
                     self.event_paused_joystick_axis_move(event.joy, event.axis,
                                                          event.value)
@@ -628,38 +668,6 @@ class Game(object):
                     for obj in self.current_room.objects:
                         obj.event_paused_joystick_axis_move(
                             event.joy, event.axis, event.value)
-                elif event.type == pygame.JOYBALLMOTION:
-                    # Limited support for trackballs by pretending
-                    # they're axes.  Since they're acting like axes,
-                    # they must be in the range [-1,1] unless the
-                    # special variable real_trackballs is True.
-                    n = (self._joysticks[event.joy].get_numaxes() +
-                         2 * event.ball)
-
-                    if sge.real_trackballs:
-                        xvalue = event.rel[0]
-                        yvalue = event.rel[1]
-                    else:
-                        xvalue = min(max(-1, event.rel[0]), 1)
-                        yvalue = min(max(-1, event.rel[1]), 1)
-
-                    # x-axis
-                    self.event_paused_joystick_axis_move(event.joy, n,
-                                                         xvalue)
-                    self.current_room.event_paused_joystick_axis_move(
-                        event.joy, n, xvalue)
-                    for obj in self.current_room.objects:
-                        obj.event_paused_joystick_axis_move(event.joy, n,
-                                                            xvalue)
-
-                    # y-axis
-                    self.event_paused_joystick_axis_move(event.joy, n + 1,
-                                                         yvalue)
-                    self.current_room.event_paused_joystick_axis_move(
-                        event.joy, n + 1, yvalue)
-                    for obj in self.current_room.objects:
-                        obj.event_paused_joystick_axis_move(
-                            event.joy, n + 1, yvalue)
                 elif event.type == pygame.JOYHATMOTION:
                     self.event_paused_joystick_hat_move(event.joy, event.hat,
                                                         *event.value)
@@ -668,6 +676,14 @@ class Game(object):
                     for obj in self.current_room.objects:
                         obj.event_paused_joystick_hat_move(
                             event.joy, event.hat, *event.value)
+                elif event.type == pygame.JOYBALLMOTION:
+                    self.event_paused_joystick_trackball_move(
+                        event.joy, event.ball, *event.rel)
+                    self.current_room.event_paused_joystick_trackball_move(
+                        event.joy, event.ball, *event.rel)
+                    for obj in self.current_room.objects:
+                        obj.event_paused_joystick_trackball_move(
+                            event.joy, event.ball, *event.rel)
                 elif event.type == pygame.JOYBUTTONDOWN:
                     self.event_paused_joystick_button_press(event.joy,
                                                             event.button)
@@ -685,9 +701,13 @@ class Game(object):
                         obj.event_paused_joystick_button_release(event.joy,
                                                                  event.button)
                 elif event.type == pygame.QUIT:
+                    if sge.DEBUG:
+                        print('Quit requested by the system.')
                     self.current_room.event_paused_close()
                     self.event_paused_close()
                 elif event.type == pygame.VIDEORESIZE:
+                    if sge.DEBUG:
+                        print('Video resize detected.')
                     self._window_width = event.w
                     self._window_height = event.h
                     self._set_mode()
@@ -748,7 +768,9 @@ class Game(object):
     def event_key_press(self, key):
         """Key press event.
 
-        ``key`` is the key that was pressed.
+        ``key`` indicates the identifier string of the key that was
+        pressed; see the file KEYS for a full list of key identifier
+        strings.
 
         """
         pass
@@ -756,7 +778,9 @@ class Game(object):
     def event_key_release(self, key):
         """Key release event.
 
-        ``key`` is the key that was pressed.
+        ``key`` indicates the identifier string of the key that was
+        released; see the file KEYS for a full list of key identifier
+        strings.
 
         """
         pass
@@ -772,9 +796,15 @@ class Game(object):
     def event_mouse_button_press(self, button):
         """Mouse button press event.
 
-        ``button`` is the number of the mouse button that was pressed;
-        these numbers may vary by implementation, so MOUSE_BUTTON_*
-        constants should be used.
+        ``button`` is the mouse button that was pressed.  It can be any
+        of the following values:
+            "left"
+            "right"
+            "middle"
+            "wheel_up"
+            "wheel_down"
+            "wheel_left"
+            "wheel_right"
 
         """
         pass
@@ -782,9 +812,15 @@ class Game(object):
     def event_mouse_button_release(self, button):
         """Mouse button release event.
 
-        ``button`` is the number of the mouse button that was released;
-        these numbers may vary by implementation, so MOUSE_BUTTON_*
-        constants should be used.
+        ``button`` is the mouse button that was released.  It can be any
+        of the following values:
+            "left"
+            "right"
+            "middle"
+            "wheel_up"
+            "wheel_down"
+            "wheel_left"
+            "wheel_right"
 
         """
         pass
@@ -819,6 +855,20 @@ class Game(object):
         """
         pass
 
+    def event_joystick_trackball_move(self, joystick, ball, x, y):
+        """Joystick trackball move event.
+
+        ``joystick`` indicates the number of the joystick, where 0 is
+        the first joystick.  ``ball`` indicates the number of the
+        trackball, where 0 is the first trackball.  ``x`` and ``y``
+        indicate the relative movement of the trackball.
+
+        Support for joysticks in Stellar Game Engine implementations is
+        optional.
+
+        """
+        pass
+
     def event_joystick_button_press(self, joystick, button):
         """Joystick button press event.
 
@@ -877,7 +927,8 @@ class Game(object):
     def event_paused_key_press(self, key):
         """Key press event when paused.
 
-        See Game.event_key_press.__doc__ for more information.
+        See the documentation for sge.Game.event_key_press for more
+        information.
 
         """
         pass
@@ -885,7 +936,8 @@ class Game(object):
     def event_paused_key_release(self, key):
         """Key release event when paused.
 
-        See Game.event_key_release.__doc__ for more information.
+        See the documentation for sge.Game.event_key_release for more
+        information.
 
         """
         pass
@@ -893,7 +945,8 @@ class Game(object):
     def event_paused_mouse_move(self, x, y):
         """Mouse move event when paused.
 
-        See Game.event_mouse_move.__doc__ for more information.
+        See the documentation for sge.Game.event_mouse_move for more
+        information.
 
         """
         pass
@@ -901,7 +954,8 @@ class Game(object):
     def event_paused_mouse_button_press(self, button):
         """Mouse button press event when paused.
 
-        See Game.event_mouse_button_press.__doc__ for more information.
+        See the documentation for sge.Game.event_mouse_button_press for
+        more information.
 
         """
         pass
@@ -909,8 +963,8 @@ class Game(object):
     def event_paused_mouse_button_release(self, button):
         """Mouse button release event when paused.
 
-        See Game.event_mouse_button_release.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_mouse_button_release
+        for more information.
 
         """
         pass
@@ -918,7 +972,8 @@ class Game(object):
     def event_paused_joystick_axis_move(self, joystick, axis, value):
         """Joystick axis move event when paused.
 
-        See Game.event_joystick_axis_move.__doc__ for more information.
+        See the documentation for sge.Game.event_joystick_axis_move for
+        more information.
 
         """
         pass
@@ -926,7 +981,17 @@ class Game(object):
     def event_paused_joystick_hat_move(self, joystick, hat, x, y):
         """Joystick HAT move event when paused.
 
-        See Game.event_joystick_hat_move.__doc__ for more information.
+        See the documentation for sge.Game.event_joystick_hat_move for
+        more information.
+
+        """
+        pass
+
+    def event_paused_joystick_trackball_move(self, joystick, ball, x, y):
+        """Joystick trackball move event when paused.
+
+        See the documentation for sge.Game.event_joystick_trackball_move
+        for more information.
 
         """
         pass
@@ -934,8 +999,8 @@ class Game(object):
     def event_paused_joystick_button_press(self, joystick, button):
         """Joystick button press event when paused.
 
-        See Game.event_joystick_button_press.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_button_press
+        for more information.
 
         """
         pass
@@ -943,8 +1008,8 @@ class Game(object):
     def event_paused_joystick_button_release(self, joystick, button):
         """Joystick button release event when paused.
 
-        See Game.event_joystick_button_release.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_button_release
+        for more information.
 
         """
         pass
@@ -952,7 +1017,8 @@ class Game(object):
     def event_paused_close(self):
         """Close event (e.g. close button) when paused.
 
-        See Game.event_close.__doc__ for more information.
+        See the documentation for sge.Game.event_close for more
+        information.
 
         """
         pass
