@@ -68,6 +68,7 @@ class Room(object):
         event_mouse_button_release: Mouse button release event.
         event_joystick_axis_move: Joystick axis move event.
         event_joystick_hat_move: Joystick HAT move event.
+        event_joystick_trackball_move: Joystick trackball move event.
         event_joystick_button_press: Joystick button press event.
         event_joystick_button_release: Joystick button release event.
         event_close: Close event (e.g. close button).  It is always
@@ -83,6 +84,7 @@ class Room(object):
         event_paused_mouse_button_release
         event_paused_joystick_axis_move
         event_paused_joystick_hat_move
+        event_paused_joystick_trackball_move 
         event_paused_joystick_button_press
         event_paused_joystick_button_release
         event_paused_close
@@ -151,9 +153,10 @@ class Room(object):
             new_objects = list(self.objects)
             new_objects.append(obj)
             self.objects = tuple(new_objects)
-            sge.game._pygame_sprites.add(obj._pygame_sprite, layer=obj.z)
-            if sge.game.current_room is self and self._started:
-                obj.event_create()
+            if self is sge.game.current_room:
+                sge.game._pygame_sprites.add(obj._pygame_sprite, layer=obj.z)
+                if self._started:
+                    obj.event_create()
 
     def start(self):
         """Start the room.
@@ -216,6 +219,196 @@ class Room(object):
         else:
             sge.game.end()
 
+    def project_dot(self, x, y, z, color):
+        """Project a single-pixel dot onto the room.
+
+        ``x`` and ``y`` indicate the location in the room to project the
+        dot.  ``z`` indicates the Z-axis position of the projection in
+        the room.  ``color`` indicates the color of the dot.
+
+        """
+        sprite = sge.Sprite(None, 1, 1)
+        sprite.draw_dot(0, 0, color)
+        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False)
+        self.add(p)
+
+    def project_line(self, x1, y1, x2, y2, z, color, thickness=1,
+                     anti_alias=False):
+        """Project a line segment onto the room.
+
+        ``x1``, ``y1``, ``x2``, and ``y2`` indicate the location in the
+        room of the points between which to project the line segment.
+        ``z`` indicates the Z-axis position of the projection in the
+        room.  ``color`` indicates the color of the line segment.
+        ``thickness`` indicates the thickness of the line segment in
+        pixels.  ``anti_alias`` indicates whether or not anti-aliasing
+        should be used.
+
+        Support for anti-aliasing is optional in Stellar Game Engine
+        implementations.  If the implementation used does not support
+        anti-aliasing, this method will act like ``anti_alias`` is
+        False.
+
+        """
+        thickness = abs(thickness)
+        x = min(x1, x2) - thickness // 2
+        y = min(y1, y2) - thickness // 2
+        w = abs(x2 - x1) + thickness
+        h = abs(y2 - y1) + thickness
+        x1 -= x
+        y1 -= y
+        x2 -= x
+        y2 -= y
+        sprite = sge.Sprite(None, w, h)
+        sprite.draw_line(x1, y1, x2, y2, color, thickness, anti_alias)
+        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False)
+        self.add(p)
+
+    def project_rectangle(self, x, y, z, width, height, fill=None,
+                          outline=None, outline_thickness=1):
+        """Project a rectangle onto the room.
+
+        ``x`` and ``y`` indicate the location in the room to position
+        the top-left corner of the rectangle.  ``z`` indicates the
+        Z-axis position of the projection in the room.  ``width`` and
+        ``height`` indicate the size of the rectangle.  ``fill``
+        indicates the color of the fill of the rectangle; set to None
+        for no fill.  ``outline`` indicates the color of the outline of
+        the rectangle; set to None for no outline.
+        ``outline_thickness`` indicates the thickness of the outline in
+        pixels (ignored if there is no outline).
+
+        """
+        outline_thickness = abs(outline_thickness)
+        draw_x = outline_thickness // 2
+        draw_y = outline_thickness // 2
+        x -= draw_x
+        y -= draw_y
+        w = width + outline_thickness
+        h = height + outline_thickness
+        sprite = sge.Sprite(None, w, h)
+        sprite.draw_rectangle(draw_x, draw_y, w, h, fill, outline,
+                              outline_thickness)
+        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False)
+        self.add(p)
+
+    def project_ellipse(self, x, y, z, width, height, fill=None, outline=None,
+                        outline_thickness=1, anti_alias=False):
+        """Project an ellipse onto the room.
+
+        ``x`` and ``y`` indicate the location in the room to position
+        the top-left corner of the imaginary rectangle containing the
+        ellipse.  ``z`` indicates the Z-axis position of the projection
+        in the room.  ``width`` and ``height`` indicate the size of the
+        ellipse.  ``fill`` indicates the color of the fill of the
+        ellipse; set to None for no fill.  ``outline`` indicates the
+        color of the outline of the ellipse; set to None for no outline.
+        ``outline_thickness`` indicates the thickness of the outline in
+        pixels (ignored if there is no outline).  ``anti_alias``
+        indicates whether or not anti-aliasing should be used on the
+        outline.
+
+        Support for anti-aliasing is optional in Stellar Game Engine
+        implementations.  If the implementation used does not support
+        anti-aliasing, this method will act like ``anti_alias`` is
+        False.
+
+        """
+        outline_thickness = abs(outline_thickness)
+        draw_x = outline_thickness // 2
+        draw_y = outline_thickness // 2
+        x -= draw_x
+        y -= draw_y
+        w = width + outline_thickness
+        h = height + outline_thickness
+        sprite = sge.Sprite(None, w, h)
+        sprite.draw_ellipse(draw_x, draw_y, w, h, fill, outline,
+                            outline_thickness)
+        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False)
+        self.add(p)
+
+    def project_circle(self, x, y, z, radius, fill=None, outline=None,
+                       outline_thickness=1, anti_alias=False):
+        """Project a circle onto the room.
+
+        ``x`` and ``y`` indicate the location in the room to position
+        the center of the circle.  ``z`` indicates the Z-axis position
+        of the projection in the room.  ``radius`` indicates the radius
+        of the circle in pixels.  ``fill`` indicates the color of the
+        fill of the circle; set to None for no fill.  ``outline``
+        indicates the color of the outline of the circle; set to None
+        for no outline.  ``outline_thickness`` indicates the thickness
+        of the outline in pixels (ignored if there is no outline).
+        ``anti_alias`` indicates whether or not anti-aliasing should be
+        used on the outline.
+
+        Support for anti-aliasing is optional in Stellar Game Engine
+        implementations.  If the implementation used does not support
+        anti-aliasing, this method will act like ``anti_alias`` is
+        False.
+
+        """
+        outline_thickness = abs(outline_thickness)
+        xy = radius + outline_thickness // 2
+        wh = 2 * radius + outline_thickness
+        sprite = sge.Sprite(None, wh, wh)
+        sprite.draw_circle(xy, xy, radius, fill, outline, outline_thickness,
+                           anti_alias)
+        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False)
+        self.add(p)
+
+    def project_sprite(self, sprite, image, x, y, z):
+        """Project a sprite onto the room.
+
+        ``sprite`` indicates the sprite to draw.  ``image`` indicates
+        the frame of the sprite to draw, where 0 is the first frame.
+        ``x`` and ``y`` indicate the location in the room to position
+        the sprite.  ``z`` indicates the Z-axis position of the
+        projection in the room.
+
+        """
+        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False,
+                        image_index=image, image_fps=0)
+        self.add(p)
+
+    def project_text(self, font, text, x, y, z, width=None, height=None,
+                    color="black", halign=sge.ALIGN_LEFT, valign=sge.ALIGN_TOP,
+                    anti_alias=True):
+        """Project text onto the room.
+
+        ``font`` indicates the font to use for the text.  ``text``
+        indicates the text to project.  ``x`` and ``y`` indicate the
+        location in the room to project the text.  ``width`` and
+        ``height`` indicate the size of the imaginary box the text is
+        projected in; set to None for no imaginary box.  ``color``
+        indicates the color of the text.  ``halign`` indicates the
+        horizontal alignment of the text and can be ALIGN_LEFT,
+        ALIGN_CENTER, or ALIGN_RIGHT.  ``valign`` indicates the vertical
+        alignment and can be ALIGN_TOP, ALIGN_MIDDLE, or ALIGN_BOTTOM.
+        ``anti_alias`` indicates whether or not anti-aliasing should be
+        used.
+
+        If the text does not fit into the imaginary box specified, the
+        text that doesn't fit will be cut off at the bottom if valign is
+        ALIGN_TOP, the top if valign is ALIGN_BOTTOM, or equally the top
+        and bottom if valign is ALIGN_MIDDLE.
+
+        Support for anti-aliasing is optional in Stellar Game Engine
+        implementations.  If the implementation used does not support
+        anti-aliasing, this function will act like ``anti_alias`` is False.
+
+        """
+        w, h = font.get_size(text, width, height)
+        draw_x = {sge.ALIGN_LEFT: 0, sge.ALIGN_CENTER: w / 2,
+                  sge.ALIGN_RIGHT: w}.setdefault(halign, w / 2)
+        draw_y = {sge.ALIGN_TOP: 0, sge.ALIGN_MIDDLE: h / 2,
+                  sge.ALIGN_BOTTOM: h}.setdefault(valign, h / 2)
+        sprite = sge.Sprite(None, w, h)
+        sprite.draw_text(font, text, draw_x, draw_y, width, height, color,
+                         halign, valign, anti_alias)
+        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False)
+        self.add(p)
+
     def event_room_start(self):
         """Room start event.
 
@@ -238,8 +431,8 @@ class Room(object):
     def event_step(self, time_passed):
         """Room step event.
 
-        Called once each frame.  ``time_passed`` is the number of
-        milliseconds that have passed during the last frame. 
+        See the documentation for sge.Game.event_step for more
+        information.
 
         """
         pass
@@ -247,7 +440,8 @@ class Room(object):
     def event_key_press(self, key):
         """Key press event.
 
-        ``key`` is the key that was pressed.
+        See the documentation for sge.Game.event_key_press for more
+        information.
 
         """
         pass
@@ -255,7 +449,8 @@ class Room(object):
     def event_key_release(self, key):
         """Key release event.
 
-        ``key`` is the key that was pressed.
+        See the documentation for sge.Game.event_key_release for more
+        information.
 
         """
         pass
@@ -263,7 +458,8 @@ class Room(object):
     def event_mouse_move(self, x, y):
         """Mouse move event.
 
-        ``x`` and ``y`` indicate the relative movement of the mouse.
+        See the documentation for sge.Game.event_mouse_move for more
+        information.
 
         """
         pass
@@ -271,9 +467,8 @@ class Room(object):
     def event_mouse_button_press(self, button):
         """Mouse button press event.
 
-        ``button`` is the number of the mouse button that was pressed;
-        these numbers may vary by implementation, so MOUSE_BUTTON_*
-        constants should be used.
+        See the documentation for sge.Game.event_mouse_button_press for
+        more information.
 
         """
         pass
@@ -281,9 +476,8 @@ class Room(object):
     def event_mouse_button_release(self, button):
         """Mouse button release event.
 
-        ``button`` is the number of the mouse button that was released;
-        these numbers may vary by implementation, so MOUSE_BUTTON_*
-        constants should be used.
+        See the documentation for sge.Game.event_mouse_button_release
+        for more information.
 
         """
         pass
@@ -291,14 +485,8 @@ class Room(object):
     def event_joystick_axis_move(self, joystick, axis, value):
         """Joystick axis move event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``axis`` is the number of the axis, where 0 is the
-        first axis.  ``value`` is the tilt of the axis, where 0 is in
-        the center, -1 is tilted all the way to the left or up, and 1 is
-        tilted all the way to the right or down.
-
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        See the documentation for sge.Game.event_joystick_axis_move for
+        more information.
 
         """
         pass
@@ -306,14 +494,17 @@ class Room(object):
     def event_joystick_hat_move(self, joystick, hat, x, y):
         """Joystick HAT move event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``hat`` is the number of the HAT, where 0 is the
-        first HAT.  ``x`` and ``y`` indicate the position of the HAT,
-        where 0 is in the center, -1 is left or up, and 1 is right or
-        down.
+        See the documentation for sge.Game.event_joystick_hat_move for
+        more information.
 
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        """
+        pass
+
+    def event_joystick_trackball_move(self, joystick, ball, x, y):
+        """Joystick trackball move event.
+
+        See the documentation for sge.Game.event_joystick_trackball_move
+        for more information.
 
         """
         pass
@@ -321,12 +512,8 @@ class Room(object):
     def event_joystick_button_press(self, joystick, button):
         """Joystick button press event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``button`` is the number of the button pressed, where
-        0 is the first button.
-
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        See the documentation for sge.Game.event_joystick_button_press
+        for more information.
 
         """
         pass
@@ -334,12 +521,8 @@ class Room(object):
     def event_joystick_button_release(self, joystick, button):
         """Joystick button release event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``button`` is the number of the button pressed, where
-        0 is the first button.
-
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        See the documentation for sge.Game.event_joystick_button_release
+        for more information.
 
         """
         pass
@@ -347,8 +530,9 @@ class Room(object):
     def event_close(self):
         """Close event (e.g. close button).
 
-        It is always called before any game close events occurring at
-        the same time.
+        See the documentation for sge.Game.event_close for more
+        information.  This is always called before any game close events
+        occurring at the same time.
 
         """
         pass
@@ -356,7 +540,8 @@ class Room(object):
     def event_paused_key_press(self, key):
         """Key press event when paused.
 
-        See Room.event_key_press.__doc__ for more information.
+        See the documentation for sge.Game.event_key_press for more
+        information.
 
         """
         pass
@@ -364,7 +549,8 @@ class Room(object):
     def event_paused_key_release(self, key):
         """Key release event when paused.
 
-        See Room.event_key_release.__doc__ for more information.
+        See the documentation for sge.Game.event_key_release for more
+        information.
 
         """
         pass
@@ -372,7 +558,8 @@ class Room(object):
     def event_paused_mouse_move(self, x, y):
         """Mouse move event when paused.
 
-        See Room.event_mouse_move.__doc__ for more information.
+        See the documentation for sge.Game.event_mouse_move for more
+        information.
 
         """
         pass
@@ -380,7 +567,8 @@ class Room(object):
     def event_paused_mouse_button_press(self, button):
         """Mouse button press event when paused.
 
-        See Room.event_mouse_button_press.__doc__ for more information.
+        See the documentation for sge.Game.event_mouse_button_press for
+        more information.
 
         """
         pass
@@ -388,8 +576,8 @@ class Room(object):
     def event_paused_mouse_button_release(self, button):
         """Mouse button release event when paused.
 
-        See Room.event_mouse_button_release.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_mouse_button_release
+        for more information.
 
         """
         pass
@@ -397,7 +585,8 @@ class Room(object):
     def event_paused_joystick_axis_move(self, joystick, axis, value):
         """Joystick axis move event when paused.
 
-        See Room.event_joystick_axis_move.__doc__ for more information.
+        See the documentation for sge.Game.event_joystick_axis_move for
+        more information.
 
         """
         pass
@@ -405,7 +594,17 @@ class Room(object):
     def event_paused_joystick_hat_move(self, joystick, hat, x, y):
         """Joystick HAT move event when paused.
 
-        See Room.event_joystick_hat_move.__doc__ for more information.
+        See the documentation for sge.Game.event_joystick_hat_move for
+        more information.
+
+        """
+        pass
+
+    def event_paused_joystick_trackball_move(self, joystick, ball, x, y):
+        """Joystick trackball move event when paused.
+
+        See the documentation for sge.Game.event_joystick_trackball_move
+        for more information.
 
         """
         pass
@@ -413,8 +612,8 @@ class Room(object):
     def event_paused_joystick_button_press(self, joystick, button):
         """Joystick button press event when paused.
 
-        See Room.event_joystick_button_press.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_button_press
+        for more information.
 
         """
         pass
@@ -422,8 +621,8 @@ class Room(object):
     def event_paused_joystick_button_release(self, joystick, button):
         """Joystick button release event when paused.
 
-        See Room.event_joystick_button_release.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_button_release
+        for more information.
 
         """
         pass
@@ -431,7 +630,8 @@ class Room(object):
     def event_paused_close(self):
         """Close event (e.g. close button) when paused.
 
-        See Room.event_close.__doc__ for more information.
+        See the documentation for sge.Room.event_close for more
+        information.
 
         """
         pass
@@ -463,3 +663,27 @@ class Room(object):
 
         for obj in self.objects:
             obj._reset()
+
+
+class _Projection(sge.StellarClass):
+
+    # Object which destroys itself after being shown for one frame.
+
+    def event_create(self):
+        self.detects_collisions = False
+        # FIXME: The alarm is set to 2 because the object doesn't show
+        # up on the first frame it is created.  It would be better if it
+        # did because depending on delta timing, as it is now, the
+        # object can end up not showing up, resulting in at best a
+        # flicker effect and at worst important visuals being missing.
+        # Waiting longer than two frames to destroy the object is not
+        # a solution because this would result in partial transparency
+        # being ruined.
+        self.set_alarm('destroy', 2)
+
+    def event_alarm(self, alarm_id):
+        self.destroy()
+
+    def event_destroy(self):
+        if self.sprite is not None:
+            del sge.game.sprites[self.sprite.name]
