@@ -41,6 +41,11 @@ class StellarClass(object):
             Sprite object or None when read, it can also be set to the
             ID of a sprite.
         visible: Whether or not the object should be drawn.
+        active: Whether or not the object should be active.  If this is
+            False, all normal occurances (including events) will not
+            execute.  Only collision events, destroy events, and events
+            with names that start with "event_inactive_" will be
+            executed.
         detects_collisions: Whether or not the object should be involved
             in collisions.
         bbox_x: The horizontal location of the top-left corner of the
@@ -118,6 +123,7 @@ class StellarClass(object):
         event_mouse_button_release: Mouse button release event.
         event_joystick_axis_move: Joystick axis move event.
         event_joystick_hat_move: Joystick HAT move event.
+        event_joystick_trackball_move: Joystick trackball move event.
         event_joystick_button_press: Joystick button press event.
         event_joystick_button_release: Joystick button release event.
         event_collision: Middle/default collision event.
@@ -126,6 +132,21 @@ class StellarClass(object):
         event_collision_top: Top collision event.
         event_collision_bottom: Bottom collision event.
 
+    The following alternative events are executed when the object is
+    inactive (i.e. the attribute ``active`` is set to False) in place of
+    the corresponding normal events:
+        event_inactive_step
+        event_inactive_key_press
+        event_inactive_key_release
+        event_inactive_mouse_move
+        event_inactive_mouse_button_press
+        event_inactive_mouse_button_release
+        event_inactive_joystick_axis_move
+        event_inactive_joystick_hat_move
+        event_inactive_joystick_trackball_move
+        event_inactive_joystick_button_press
+        event_inactive_joystick_button_release
+
     The following alternative events are executed when the game is
     paused in place of the corresponding normal events:
         event_paused_key_press
@@ -135,18 +156,19 @@ class StellarClass(object):
         event_paused_mouse_button_release
         event_paused_joystick_axis_move
         event_paused_joystick_hat_move
+        event_paused_joystick_trackball_move
         event_paused_joystick_button_press
         event_paused_joystick_button_release
 
     """
 
     def __init__(self, x, y, z, id_=None, sprite=None, visible=True,
-                 detects_collisions=True, bbox_x=None, bbox_y=None,
-                 bbox_width=None, bbox_height=None, collision_ellipse=False,
-                 collision_precise=False, xvelocity=0, yvelocity=0,
-                 image_index=0, image_fps=0, image_xscale=1, image_yscale=1,
-                 image_rotation=0, image_alpha=255, image_blend=None,
-                 **kwargs):
+                 active=True, detects_collisions=True, bbox_x=None,
+                 bbox_y=None, bbox_width=None, bbox_height=None,
+                 collision_ellipse=False, collision_precise=False,
+                 xvelocity=0, yvelocity=0, image_index=0, image_fps=0,
+                 image_xscale=1, image_yscale=1, image_rotation=0,
+                 image_alpha=255, image_blend=None, **kwargs):
         """Create a new StellarClass object.
 
         Arguments set the properties of the object.  See
@@ -226,8 +248,8 @@ class StellarClass(object):
     def event_step(self, time_passed):
         """Step event.
 
-        Called once each frame.  ``time_passed`` is the number of
-        milliseconds that have passed during the last frame.
+        See the documentation for sge.Game.event_step for more
+        information.
 
         """
         pass
@@ -252,7 +274,8 @@ class StellarClass(object):
     def event_key_press(self, key):
         """Key press event.
 
-        ``key`` is the key that was pressed.
+        See the documentation for sge.Game.event_key_press for more
+        information.
 
         """
         pass
@@ -260,7 +283,8 @@ class StellarClass(object):
     def event_key_release(self, key):
         """Key release event.
 
-        ``key`` is the key that was pressed.
+        See the documentation for sge.Game.event_key_release for more
+        information.
 
         """
         pass
@@ -268,7 +292,8 @@ class StellarClass(object):
     def event_mouse_move(self, x, y):
         """Mouse move event.
 
-        ``x`` and ``y`` indicate the relative movement of the mouse.
+        See the documentation for sge.Game.event_mouse_move for more
+        information.
 
         """
         pass
@@ -276,9 +301,8 @@ class StellarClass(object):
     def event_mouse_button_press(self, button):
         """Mouse button press event.
 
-        ``button`` is the number of the mouse button that was pressed;
-        these numbers may vary by implementation, so MOUSE_BUTTON_*
-        constants should be used.
+        See the documentation for sge.Game.event_mouse_button_press for
+        more information.
 
         """
         pass
@@ -286,9 +310,8 @@ class StellarClass(object):
     def event_mouse_button_release(self, button):
         """Mouse button release event.
 
-        ``button`` is the number of the mouse button that was released;
-        these numbers may vary by implementation, so MOUSE_BUTTON_*
-        constants should be used.
+        See the documentation for sge.Game.event_mouse_button_release
+        for more information.
 
         """
         pass
@@ -296,14 +319,8 @@ class StellarClass(object):
     def event_joystick_axis_move(self, joystick, axis, value):
         """Joystick axis move event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``axis`` is the number of the axis, where 0 is the
-        first axis.  ``value`` is the tilt of the axis, where 0 is in
-        the center, -1 is tilted all the way to the left or up, and 1 is
-        tilted all the way to the right or down.
-
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        See the documentation for sge.Game.event_joystick_axis_move for
+        more information.
 
         """
         pass
@@ -311,14 +328,17 @@ class StellarClass(object):
     def event_joystick_hat_move(self, joystick, hat, x, y):
         """Joystick HAT move event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``hat`` is the number of the HAT, where 0 is the
-        first HAT.  ``x`` and ``y`` indicate the position of the HAT,
-        where 0 is in the center, -1 is left or up, and 1 is right or
-        down.
+        See the documentation for sge.Game.event_joystick_hat_move for
+        more information.
 
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        """
+        pass
+
+    def event_joystick_trackball_move(self, joystick, ball, x, y):
+        """Joystick trackball move event.
+
+        See the documentation for sge.Game.event_joystick_trackball_move
+        for more information.
 
         """
         pass
@@ -326,12 +346,8 @@ class StellarClass(object):
     def event_joystick_button_press(self, joystick, button):
         """Joystick button press event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``button`` is the number of the button pressed, where
-        0 is the first button.
-
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        See the documentation for sge.Game.event_joystick_button_press
+        for more information.
 
         """
         pass
@@ -339,12 +355,8 @@ class StellarClass(object):
     def event_joystick_button_release(self, joystick, button):
         """Joystick button release event.
 
-        ``joystick`` is the number of the joystick, where 0 is the first
-        joystick.  ``button`` is the number of the button pressed, where
-        0 is the first button.
-
-        Support for joysticks in Stellar Game Engine implementations is
-        optional.
+        See the documentation for sge.Game.event_joystick_button_release
+        for more information.
 
         """
         pass
@@ -369,10 +381,120 @@ class StellarClass(object):
         """Bottom collision event."""
         self.event_collision(other)
 
+    def event_inactive_step(self, time_passed):
+        """Step event when this object is inactive.
+
+        See the documentation for sge.Game.event_step for more
+        information.  The object is considered to be inactive when its
+        ``active`` attribute is False.
+
+        """
+
+    def event_inactive_key_press(self, key):
+        """Key press event when this object is inactive.
+
+        See the documentation for sge.Game.event_key_press for more
+        information.  The object is considered to be inactive when its
+        ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_key_release(self, key):
+        """Key release event when this object is inactive.
+
+        See the documentation for sge.Game.event_key_release for more
+        information.  The object is considered to be inactive when its
+        ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_mouse_move(self, x, y):
+        """Mouse move event when this object is inactive.
+
+        See the documentation for sge.Game.event_mouse_move for more
+        information.  The object is considered to be inactive when its
+        ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_mouse_button_press(self, button):
+        """Mouse button press event when this object is inactive.
+
+        See the documentation for sge.Game.event_mouse_button_press for
+        more information.  The object is considered to be inactive when
+        its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_mouse_button_release(self, button):
+        """Mouse button release event when this object is inactive.
+
+        See the documentation for sge.Game.event_mouse_button_release
+        for more information.  The object is considered to be inactive
+        when its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_joystick_axis_move(self, joystick, axis, value):
+        """Joystick axis move event when this object is inactive.
+
+        See the documentation for sge.Game.event_joystick_axis_move for
+        more information.  The object is considered to be inactive when
+        its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_joystick_hat_move(self, joystick, hat, x, y):
+        """Joystick HAT move event when this object is inactive.
+
+        See the documentation for sge.Game.event_joystick_hat_move for
+        more information.  The object is considered to be inactive when
+        its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_joystick_trackball_move(self, joystick, ball, x, y):
+        """Joystick trackball move event when this object is inactive.
+
+        See the documentation for sge.Game.event_joystick_trackball_move
+        for more information.  The object is considered to be inactive
+        when its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_joystick_button_press(self, joystick, button):
+        """Joystick button press event when this object is inactive.
+
+        See the documentation for sge.Game.event_joystick_button_press
+        for more information.  The object is considered to be inactive
+        when its ``active`` attribute is False.
+
+        """
+        pass
+
+    def event_inactive_joystick_button_release(self, joystick, button):
+        """Joystick button release event when this object is inactive.
+
+        See the documentation for sge.Game.event_joystick_button_release
+        for more information.  The object is considered to be inactive
+        when its ``active`` attribute is False.
+
+        """
+        pass
+
     def event_paused_key_press(self, key):
         """Key press event when paused.
 
-        See StellarClass.event_key_press.__doc__ for more information.
+        See the documentation for sge.Game.event_key_press for more
+        information.
 
         """
         pass
@@ -380,7 +502,8 @@ class StellarClass(object):
     def event_paused_key_release(self, key):
         """Key release event when paused.
 
-        See StellarClass.event_key_release.__doc__ for more information.
+        See the documentation for sge.Game.event_key_release for more
+        information.
 
         """
         pass
@@ -388,7 +511,8 @@ class StellarClass(object):
     def event_paused_mouse_move(self, x, y):
         """Mouse move event when paused.
 
-        See StellarClass.event_mouse_move.__doc__ for more information.
+        See the documentation for sge.Game.event_mouse_move for more
+        information.
 
         """
         pass
@@ -396,8 +520,8 @@ class StellarClass(object):
     def event_paused_mouse_button_press(self, button):
         """Mouse button press event when paused.
 
-        See StellarClass.event_mouse_button_press.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_mouse_button_press for
+        more information.
 
         """
         pass
@@ -405,8 +529,8 @@ class StellarClass(object):
     def event_paused_mouse_button_release(self, button):
         """Mouse button release event when paused.
 
-        See StellarClass.event_mouse_button_release.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_mouse_button_release
+        for more information.
 
         """
         pass
@@ -414,8 +538,8 @@ class StellarClass(object):
     def event_paused_joystick_axis_move(self, joystick, axis, value):
         """Joystick axis move event when paused.
 
-        See StellarClass.event_joystick_axis_move.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_axis_move for
+        more information.
 
         """
         pass
@@ -423,8 +547,17 @@ class StellarClass(object):
     def event_paused_joystick_hat_move(self, joystick, hat, x, y):
         """Joystick HAT move event when paused.
 
-        See StellarClass.event_joystick_hat_move.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_hat_move for
+        more information.
+
+        """
+        pass
+
+    def event_paused_joystick_trackball_move(self, joystick, ball, x, y):
+        """Joystick trackball move event when paused.
+
+        See the documentation for sge.Game.event_joystick_trackball_move
+        for more information.
 
         """
         pass
@@ -432,8 +565,8 @@ class StellarClass(object):
     def event_paused_joystick_button_press(self, joystick, button):
         """Joystick button press event when paused.
 
-        See StellarClass.event_joystick_button_press.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_button_press
+        for more information.
 
         """
         pass
@@ -441,8 +574,8 @@ class StellarClass(object):
     def event_paused_joystick_button_release(self, joystick, button):
         """Joystick button release event when paused.
 
-        See StellarClass.event_joystick_button_release.__doc__ for more
-        information.
+        See the documentation for sge.Game.event_joystick_button_release
+        for more information.
 
         """
         pass
